class Accounting < ActiveRecord::Base

include NumberHelper

attr_reader :total_calls, :total_minutes, :local_to_sip_calls, :local_to_sip_minutes, :external_to_sip_calls, :external_to_sip_minutes, \
:sip_to_local_calls, :sip_to_local_minutes, :sip_to_external_calls, :sip_to_external_minutes, :sip_to_sip_calls, :sip_to_sip_minutes

def self.columns() @columns ||= []; end
def self.column(name, sql_type = nil, default = nil, null = true)
    columns << ActiveRecord::ConnectionAdapters::Column.new(name.to_s, default, sql_type.to_s, null)
end

def initialize

@total_calls = 0
@total_minutes = 0
@local_to_sip_calls = 0
@local_to_sip_minutes = 0
@external_to_sip_calls = 0
@external_to_sip_minutes = 0
@sip_to_sip_calls = 0
@sip_to_sip_minutes = 0
@sip_to_local_calls = 0
@sip_to_local_minutes = 0
@sip_to_external_calls = 0
@sip_to_external_minutes = 0

end

def get_global_accounting_data(start_time = nil, end_time = nil)

start_time ||= Time.mktime(Date.today.year, Date.today.month).to_date
end_time ||= Time.mktime(Date.today.year, Date.today.month + 1).to_date

query =	'SELECT 
			invite.from_uri, 
			invite.to_uri,
			invite.sip_callid,
			invite.timestamp as start,
			bye.timestamp as end,
			TIMESTAMPDIFF(MINUTE, invite.timestamp, bye.timestamp) as time,
			subscriber.id
		FROM acc as invite, acc as bye, subscriber
		WHERE
			invite.sip_method="INVITE" AND 
			bye.sip_method="BYE" AND
			invite.sip_callid = bye.sip_callid AND
			(subscriber.username = bye.username OR subscriber.username = invite.username) AND 
			(invite.time>="' + start_time.to_s(:mysql) + '" AND bye.time<="' + end_time.to_s(:mysql) + '")
			ORDER BY start DESC'

		result = ActiveRecord::Base.connection.execute(query)

		call_data = self.process_minutes_data(result)
		return call_data

end

def get_user_accounting_data(user, start_time = nil, end_time = nil)

start_time ||= Time.mktime(Date.today.year, Date.today.month).to_date
end_time ||= Time.mktime(Date.today.year, Date.today.month + 1).to_date

query =	'SELECT 
			invite.from_uri, 
			invite.to_uri,
			invite.sip_callid,
			invite.timestamp as start,
			bye.timestamp as end,
			TIMESTAMPDIFF(MINUTE, invite.timestamp, bye.timestamp) as time,
			subscriber.id
		FROM acc as invite, acc as bye, subscriber
		WHERE
			invite.sip_method="INVITE" AND 
			bye.sip_method="BYE" AND
			invite.sip_callid = bye.sip_callid AND
			(subscriber.username = bye.username OR subscriber.username = invite.username) AND 
			subscriber.id = ' + user.id.to_s + ' AND
			(invite.time>="' + start_time.to_s(:mysql) + '" AND bye.time<="' + end_time.to_s(:mysql) + '")
			ORDER BY start DESC'

		result = ActiveRecord::Base.connection.execute(query)
		result2 = ActiveRecord::Base.connection.execute(query)

		call_data = self.process_user_calls(result)
		other = self.process_minutes_data(result2)
		
		return call_data

end

def process_minutes_data(calls)

	user_data = Hash.new{|h,k| h[k] = Hash.new(0)}

	calls.each_hash do |call|
		
		from = sip_to_pstn(call['from_uri'])
		to = sip_to_pstn(call['to_uri'])
		time = call['time'].to_i 
		
		if time == 0 then time = 1 end
		@total_calls += 1 
		@total_minutes += time

		if is_number_sip?(from) then 

			user_data[from][:outgoing_minutes] += time 
			user_data[from][:outgoing_calls] += 1
			user_data[from][:name] = name_from_number(from) if user_data[from][:name] == 0

 			if is_number_sip?(to) then
	
				@sip_to_sip_calls += 1 
				@sip_to_sip_minutes += time

				user_data[to][:incoming_minutes] += time
				user_data[to][:incoming_calls] += 1
				user_data[to][:name] = name_from_number(to) if user_data[to][:name] == 0

			elsif is_number_local?(to)

				@sip_to_local_calls += 1
				@sip_to_local_minutes += time
				
 			elsif is_number_external?(to)

				@sip_to_external_calls += 1
				@sip_to_external_minutes += time

			end

 		elsif is_number_local?(from) then

			user_data[to][:incoming_minutes] += time
			user_data[to][:incoming_calls] += 1
			user_data[to][:name] = name_from_number(to) if user_data[to][:name] == 0

			@local_to_sip_calls += 1
			@local_to_sip_minutes += time
			

 		elsif is_number_external?(from) then

			user_data[to][:incoming_minutes] += time
			user_data[to][:incoming_calls] += 1
			user_data[to][:name] = name_from_number(to) if user_data[to][:name] == 0

			@external_to_sip_calls += 1
			@external_to_sip_minutes += time

		end

	end		

	return user_data

end

def process_user_calls(calls)

  call_data = Array.new

  calls.each_hash do |call|

	nc = Hash.new
	nc[:from] = sip_to_pstn(call['from_uri'])
	nc[:to] = sip_to_pstn(call['to_uri'])
	nc[:start_time] = call['start']
	nc[:end_time] = call['end']

	time = call['time'].to_i
	if time == 0 then time = 1 end

	nc[:length] = time.to_s

	call_data += [nc]
  end

  return call_data

end
end